# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
import logging

from odoo import _, api, fields, models
from odoo.exceptions import UserError, ValidationError

_logger = logging.getLogger(__name__)


class ScopInstance(models.Model):
    _inherit = "scop.instance"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    is_type_membership = fields.Boolean("Adhésions")
    date_membership = fields.Date(
        string="Date d'adhésion",
    )
    instance_partner_ids = fields.One2many(
        comodel_name="scop.instance.partner",
        inverse_name="instance_id",
        string="Coopératives",
        required=False,
    )
    partners_count = fields.Integer(
        string="Nombre d'adhésions",
        compute="_compute_partners_count",
    )

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    def _compute_partners_count(self):
        for r in self:
            r.partners_count = len(r.instance_partner_ids)

    # ------------------------------------------------------
    # Onchange fields
    # ------------------------------------------------------
    @api.onchange("date")
    def onchange_date(self):
        if not self.date_membership:
            self.date_membership = fields.Date.to_date(self.date)

    # ------------------------------------------------------
    # Business method
    # ------------------------------------------------------
    def create_report_adhesion(self, partners):
        """
        Generic function to create report adhésion CG
        :return:
        """
        return self.env.ref("cgscop_adhesion.cgscop_adhesion_report").report_action(
            partners
        )

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_add_coop(self):
        """
        Display wizard to attach coop to instance
        :return:
        """
        self.ensure_one()
        wizard = self.env["scop.instance.wizard"].create({"instance_id": self.id})
        return {
            "name": "Ajouter des coops",
            "type": "ir.actions.act_window",
            "view_mode": "form",
            "res_model": "scop.instance.wizard",
            "res_id": wizard.id,
            "target": "new",
        }

    def create_report_adhesion_all(self):
        """
        Create report adhésion CG ALL
        :return:
        """
        partners = self.instance_partner_ids.mapped("partner_id").filtered(
            lambda p: p.membership_status == "member"
        )
        if len(partners) < 1:
            raise UserError(_('Aucune coopérative n\'est en statut "membre"'))
        return self.create_report_adhesion(partners)

    def action_show_coop(self):
        """
        Display tree view of coop linked to instance
        :return:
        """
        self.ensure_one()
        partner_ids = self._get_partners().ids
        tree_view = self.env.ref(
            "cgscop_adhesion_instance.res_partner_instance_coops_tree"
        ).id
        return {
            "type": "ir.actions.act_window",
            "name": "Coopératives",
            "views": [[tree_view, "tree"], [False, "form"]],
            "view_mode": "tree",
            "res_model": "res.partner",
            "target": "current",
            "domain": [["id", "in", partner_ids]],
        }

    # ------------------------------------------------------
    # Validation
    # ------------------------------------------------------
    def scop_valid_cg_mass(self):
        """
        Select all coop in state positive & status "soumis cg" to be validated
        - use scop_valid_cg
        - set lf_adhesion qualified
        - log success or error
        - open wizard to send mail to all validated coops
        """
        # Check if function is called with context or not
        partners_to_validate = self.env.context.get(
            "partners_to_validate",
            self.instance_partner_ids.filtered(
                lambda p: p.state == "positive" and p.membership_status == "soumis_cg"
            ),
        )
        validated_partners = list()
        for r in partners_to_validate:
            partner = r.partner_id
            try:
                with self.env.cr.savepoint():
                    # scop_valid_cg
                    partner.with_context(origin_instance_id=self.id).scop_valid_cg()
                    # set membership stat period
                    membership_period = self.env["scop.membership.period"].search(
                        [
                            ("partner_id", "=", partner.id),
                            ("end", "=", False),
                        ],
                    )
                    if membership_period:
                        membership_period.update({"start": r.statistic_membership_date})
                    # log success
                    validated_partners.append(partner.id)
                    r.is_success = True
            except Exception as e:
                logger = "Erreur lors la validation du dossier %s : " "%s" % (
                    partner.name,
                    e.args[0],
                )
                _logger.warning(logger)
                r.is_success = False
                r.message_error = (
                    fields.datetime.now().strftime("%d/%m/%Y - %-H:%M")
                    + " - Erreur : "
                    + e.args[0]
                )
                if len(partners_to_validate) == 1:
                    raise ValidationError(logger)

    def update_membership_date(self):
        for partner in self.instance_partner_ids:
            partner.update({"statistic_membership_date": self.date_membership})
            # set membership stat period
            membership_period_ids = partner.partner_id.membership_period_ids.filtered(
                lambda p: p.start == self.date.date() and not p.end
            )
            membership_period_ids.update({"start": self.date_membership})
