# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, api, fields, models


class ScopInstancePartner(models.Model):
    _name = "scop.instance.partner"
    _description = "Statut coop par instance"

    # ------------------------------------------------------
    # Default
    # ------------------------------------------------------
    @api.model
    def _get_default_date_stat(self):
        if self.env.context.get("active_model") == "scop.instance":
            return (
                self.env["scop.instance"]
                .browse(self.env.context["active_id"])
                .date.date()
            )

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    instance_id = fields.Many2one(
        comodel_name="scop.instance", string="Instance", required=True
    )

    partner_id = fields.Many2one(
        comodel_name="res.partner", string="Coopératives", required=True
    )
    display_name = fields.Char(string="Coopérative", related="partner_id.display_name")
    city = fields.Char(string="Ville", related="partner_id.city")
    ur_id = fields.Many2one(
        comodel_name="union.regionale", string="UR", related="partner_id.ur_id"
    )
    phone = fields.Char(string="Téléphone", related="partner_id.phone")
    email = fields.Char(string="Email", related="partner_id.email")
    project_status = fields.Selection(
        string="Statut projet", related="partner_id.project_status"
    )
    membership_status = fields.Selection(
        string="Statut d'adhésion", related="partner_id.membership_status"
    )

    state = fields.Selection(
        string="Avis",
        selection=[("positive", "Favorable"), ("negative", "Défavorable")],
        default="positive",
        required=True,
    )
    amount_adhesion = fields.Float(
        string="Montant Adhésion",
    )
    comments = fields.Text("Commentaires")
    statistic_membership_date = fields.Date(
        string="Date d'adhésion",
        default=lambda self: self._get_default_date_stat(),
    )

    is_success = fields.Boolean("Succès", default=True)
    message_error = fields.Text("Message d'erreur")

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------
    """
    Contrainte : Unicité clé partner_id-instance_id
    """
    _sql_constraints = [
        (
            "uniqueness_partner_instance_key",
            "unique(partner_id, instance_id)",
            "Une coop ne peut être présente qu'une seule fois par instance",
        ),
    ]

    # ------------------------------------------------------
    # Compute
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def switch_state(self):
        self.ensure_one()
        if self.state == "positive":
            self.state = "negative"
        else:
            self.state = "positive"

    def action_free_coop_from_instance(self):
        """
        Free coop from instance from "scop_instance_partner" view
        :return:
        """
        self.ensure_one()
        # On détache la coop de l'instance
        self.partner_id.write({"instance_ids": [(3, self.id)]})
        # On supprime l'objet "statut coop par instance" associé
        self.unlink()

    def scop_valid_cg_again(self):
        return self.instance_id.with_context(
            partners_to_validate=self
        ).scop_valid_cg_mass()

    def try_push_again(self):
        message = (
            "<p>"
            + self.message_error
            + ".</p>"
            + "<p><strong>Voulez-vous tenter de valider à "
            "nouveau cette coop ?</strong></p>"
        )

        return {
            "type": "ir.actions.act_window.message",
            "title": _("Erreur de validation pour " + self.partner_id.name),
            "is_html_message": True,
            "message": _(message),
            "close_button_title": False,
            "buttons": [
                {
                    "type": "method",
                    "name": "Ok",
                    "model": self._name,
                    "method": "scop_valid_cg_again",
                    "args": [self.ids],
                    "classes": "btn-primary",
                },
                {
                    "type": "ir.actions.act_window_close",
                    "name": "Annuler",
                },
            ],
        }
